' THashtable and TKeyValuePair by CoderLaureate

Type THashtable Extends TList

	'Overrides just to kill some of the TList methods.
	Method AddLast:TLink(obj:Object)
	End Method

	Method AddFirst:TLink(obj:Object)
	End Method

	'Remove an entry from the list indexed by "Key"
	Method RemoveKey(Key:String)
		Local kv:TKeyValuePair
		kv = ContainsKey(Key)
		If kv Then 
			Super.Remove(kv)
		End If
	End Method

	'Add a Key/Value pair to the list
	'When Replace is set to true the new information
	'will replace any that already exists.
	'
	'Otherwise an exception in thrown.
	Method AddKV(KVPair:TKeyValuePair, Replace:Int = False)
		Local kv:TKeyValuePair
		kv = ContainsKey(KVPair.Key)
			If kv = Null Then
				Super.AddLast(KVPair)
			Else
				If Replace Then
					kv.Value = KVPair.Value
				Else
					Throw "Key " + KVPair.Key + " already exists."
				End If
			End If
	End Method

	'Create a KV Pair using the information passed, and then add it to the list
	Method Add(Key:String, Value:Object, Replace:Int = False)
		Local kv:TKeyValuePair = TKeyValuePair.Create(Key, Value)
		AddKV(kv, Replace)
	End Method

	'Search the list by "Key" name and return an entry if found.
	'Null is returned if not found.
	Method ContainsKey:TKeyValuePair(Key:String)
		Local kv:TKeyValuePair
		For kv = EachIn Self
			If kv.Key = Key Then
				Return kv
			End If
		Next
	End Method

	'Search the list by "Value"
	Method ContainsValue:TKeyValuePair(Value:String)
		Local kv:TKeyValuePair
		For kv:TKeyValuePair = EachIn Self
			If kv.Value = Value Then
				Return kv
			End If
		Next
	End Method

	'Change the value of an entry on the list
	'Setting 'Add' to true will create the entry if it doesn't exist.
	Method SetValue(Key:String, Value:Object, AddNew:Int = False)
		Local kv:TKeyValuePair
		kv = ContainsKey(Key)
		If kv Then
			kv.Value = Value
		Else
			If AddNew Then
				Add(Key, Value)
			End If
		End If
	End Method

	'I've been playing with the idea of being able to express
	'the Hashtable contents in a human readable form.
	'This was done mainly for testing, but it may come in handy
	'in the future.
	'
	'Perhaps the ability to "Persist" a hashtable?
	Method ToString:String()
		Local kv:TKeyValuePair
		Local conj:String = ""
		Local s:String = ""
		For kv = EachIn Self
			s = s + conj + kv.Key + "("
			If conj = "" Then conj = ", " 'put in comma separator
			Local ht:THashtable = THashtable(kv.Value)
			'If the value is another THashtable then
			'call the 'ToString' method of the THashtable.
			'This will have a "Recursive" effect.
			If ht Then
				s = s + ht.ToString()
			Else
				s = s + kv.Value.ToString()
			End If
			s = s + ")"
		Next
		Return s 
	End Method

	'Same with this...
	Method ToXmlString:String()
		Local kv:TKeyValuePair
		Local s:String = ""
		For kv = EachIn Self
			s = s + "<" + kv.Key + ">"
			Local ht:THashtable = THashtable(kv.Value)
			If ht Then
				s = s + ht.ToXmlString()
			Else
				s = s + kv.Value.ToString()
			End If
			s = s + "</" + kv.Key + ">"
		Next
		Return s
	End Method

End Type

Type TKeyValuePair

	Field Key:String
	Field Value:Object

	'Constructor (For lack of a better word)
	Function Create:TKeyValuePair(Key:String, Value:Object)
		Local kv:TKeyValuePair = New TKeyValuePair
		kv.Key = Key; kv.Value = Value;
		Return kv
	End Function

	'Override for TList (Sort) method
	Method Compare:Int(otherObject:Object)
		Local Result:Int = 0
		Local otr:TKeyValuePair = TKeyValuePair(otherObject)
		If otr Then
			If Key < otr.Key Then
				Result = -1
			Else If Key > otr.Key Then
				Result = 1
			End If
		Else
			Result = -1
		End If
		Return Result
	End Method

	'Simple output for Xml Node rendering as an Attribute
	Method ToString:String()
		Return Key + "=" + Chr$(34) + Value.ToString() + Chr$(34)
	End Method

	'Output kv pair as an xml node
	Method ToNodeString:String()
		Return "<" + key + ">" + Value.ToString() + "</" + key + ">"
	End Method

End Type



'kv:TKeyValuePair = TKeyValuePair.Create("Name", "Jim Pishlo")

'Print kv.ToString()
'Print kv.ToNodeString()
'WaitKey()