' ------------------------------------------------------------------------------------
' irr::core::aabbox3df wrappers
' ------------------------------------------------------------------------------------

Public

Type AABbox3df Extends IrrBase Final

	Method EqEq:Int(pOther:AABbox3df)
		Return IrrCore_AABbox3df_EqEq(handle,pOther.handle)
	EndMethod
	Method NotEq:Int(pOther:AABbox3df)
		Return IrrCore_AABbox3df_NotEq(handle,pOther.handle)
	EndMethod
	
	Method getEdges:Vector3df[]()
		Local edges:Vector3df[8]
		Local edgeparam:Int[8]		
		
		For Local i:Int=0 To 7
			edges[i]=Vector3df.create()
			edgeparam[i]=edges[i].handle
		Next

		IrrCore_AABbox3df_getEdges(handle,edgeparam)
		
		Return edges
	EndMethod
	
	Method intersectsWithBox:Int(pOther:AABbox3df)
		Return IrrCore_AABbox3df_intersectsWithBox(handle,pOther.handle)
	EndMethod
	Method intersectsWithLine:Int(pLine:Line3df)
		Return IrrCore_AABbox3df_intersectsWithLine(handle,pLine.handle)
	EndMethod
	Method intersectsWithLineUsingVects:Int(pLineMiddle:Vector3df,pLineVect:Vector3df,halflength:Float)
		Return IrrCore_AABbox3df_intersectsWithLineUsingVects(handle,pLineMiddle.handle,pLineVect.handle,halflength)
	EndMethod
	Method classifyPlaneRelation:Int(pPlane:Plane3df)
		Return IrrCore_AABbox3df_classifyPlaneRelation(handle,pPlane.handle)
	EndMethod
	Method getCenter:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_AABbox3df_getCenter(handle))
	EndMethod
	Method getExtent:Vector3df()
		Return Vector3df.createFromHandle(IrrCore_AABbox3df_getExtent(handle))
	EndMethod
	Method getMinEdge:Vector3df()
		' dont delete
		Return Vector3df.createFromHandle(IrrCore_AABbox3df_getMinEdge(handle),False)
	EndMethod
	Method getMaxEdge:Vector3df()
		' dont delete
		Return Vector3df.createFromHandle(IrrCore_AABbox3df_getMaxEdge(handle),False)
	EndMethod
	Method isEmpty:Int()
		Return IrrCore_AABbox3df_isEmpty(handle)
	EndMethod
	Method repair()
		IrrCore_AABbox3df_repair(handle)
	EndMethod	
	Method getInterpolated:AABbox3df(pOther:AABbox3df,d:Float)
		Return createFromHandle(IrrCore_AABbox3df_getInterpolated(handle,pOther.handle,d))
	EndMethod

	Method addInternalPoint(pPoint:Vector3df)
		IrrCore_AABbox3df_addInternalPoint(handle,pPoint.handle)
	EndMethod
	Method addInternalPointFromVals(x:Float,y:Float,z:Float)
		IrrCore_AABbox3df_addInternalPointFromVals(handle,x,y,z)
	EndMethod
	Method addInternalBox(pAddBox:AABbox3df)
		IrrCore_AABbox3df_addInternalBox(handle,pAddBox.handle)
	EndMethod
	Method resetFromVals(x:Float,y:Float,z:Float)
		IrrCore_AABbox3df_resetFromVals(handle,x,y,z)
	EndMethod
	Method resetFromBox(pInitValue:AABbox3df)
		IrrCore_AABbox3df_resetFromBox(handle,pInitValue.handle)
	EndMethod
	Method resetFromVect(pInitValue:Vector3df)
		IrrCore_AABbox3df_resetFromVect(handle,pInitValue.handle)
	EndMethod
	Method isPointInside:Int(pPoint:Vector3df)
		Return IrrCore_AABbox3df_isPointInside(handle,pPoint.handle)
	EndMethod
	Method isPointTotalInside:Int(pPoint:Vector3df)
		Return IrrCore_AABbox3df_isPointTotalInside(handle,pPoint.handle)
	EndMethod
	Method isFullInside:Int(other:AABbox3df)
		Return IrrCore_AABbox3df_isFullInside(handle,other.handle)
	EndMethod

	Method getVolume:Float()
		Return IrrCore_AABbox3df_getVolume(handle) 
	EndMethod
	
	Method getArea:Float()
		Return IrrCore_AABbox3df_getArea(handle)
	EndMethod
	
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function create:AABbox3df()
		Return createFromHandle(IrrCore_AABbox3df_new())
	EndFunction

	Function createFromBox:AABbox3df(other:AABbox3df)
		Return createFromHandle(IrrCore_AABbox3df_newFromBox(other.handle))
	EndFunction
	
	Function createFromVect:AABbox3df(pInit:Vector3df)
		Return createFromHandle(IrrCore_AABbox3df_newFromVect(pInit.handle))
	EndFunction
	
	Function createFromVectWithMax:AABbox3df(pInit:Vector3df,pMax:Vector3df)
		Return createFromHandle(IrrCore_AABbox3df_newFromVectWithMax(pInit.handle,pMax.handle))
	EndFunction
	
	Function createFromVals:AABbox3df(minx:Float,miny:Float,minz:Float,maxx:Float,maxy:Float,maxz:Float)
		Return createFromHandle(IrrCore_AABbox3df_newFromVals(minx,miny,minz,maxx,maxy,maxz))
	EndFunction

	Function createFromHandle:AABbox3df(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:AABbox3df=New AABbox3df
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_AABbox3df_del(handle)
	EndMethod		
	
EndType