Rem
* Example 005 User Interface

This tutorial shows how To use the built in User Interface of
the Irrlicht Engine. It will give a brief overview And show
how To Create And use windows, buttons, scroll bars, static
texts, And list boxes.

As always, we Include the header files, And use the irrlicht
namespaces. We also store a pointer To the Irrlicht device,
a counter variable For changing the creation position of a window,
And a pointer To a listbox.
EndRem
SuperStrict
Framework Irrlicht.Core

' Declare a structure To hold some context For the event receiver so that it
' has it available inside its OnEvent() Method.
Type SAppContext
	Field device:IrrlichtDevice
	Field counter:Int
	Field listbox:IGUIListBox
EndType

' Define some values that we'll use to identify individual GUI controls.
Const GUI_ID_QUIT_BUTTON:Int = 101
Const GUI_ID_NEW_WINDOW_BUTTON:Int = 102
Const GUI_ID_FILE_OPEN_BUTTON:Int = 103
Const GUI_ID_TRANSPARENCY_SCROLL_BAR:Int = 104

Rem
The Event Receiver is Not only capable of getting keyboard And
mouse Input events, but also events of the graphical user interface
(gui). There are events For almost everything: Button click,
Listbox selection change, events that say that a element was hovered
And so on. To be able To react To some of these events, we Create
an event receiver.
We only react To gui events, And If it's such an event, we get the
id of the caller (the gui element which caused the event) And get
the pointer To the gui environment.
EndRem
Type MyEventReceiver Extends IEventReceiver

	Field Context:SAppContext

	Method OnEvent:Int(event:SEvent)

		If (event.getEventType()=EET_GUI_EVENT)
			Local id:Int = event.getGUIEventCaller().getID()
			Local env:IGUIEnvironment = Context.device.getGUIEnvironment()

			Local evtype:Int=event.getGUIEventType()
			
			Select evtype

			Rem
			If a scrollbar changed its scroll position, And it is
			'our' scrollbar (the one with id GUI_ID_TRANSPARENCY_SCROLL_BAR), then we change
			the transparency of all gui elements. This is a very
			easy task: There is a skin Object, in which all color
			settings are stored. We simply go through all colors
			stored in the skin And change their alpha value.
			EndRem
			Case EGET_SCROLL_BAR_CHANGED

				If id = GUI_ID_TRANSPARENCY_SCROLL_BAR
					Local pos:Int = IGUIScrollBar(event.getGUIEventCaller()).getPos()

					For Local i:Int = 0 Until EGDC_COUNT
						Local col:SColor = env.getSkin().GetColor(i)
						col.SetAlpha(pos)
						env.getSkin().SetColor(i,col)
					Next
				EndIf
			Rem
			If a button was clicked, it could be one of 'our'
			three buttons. If it is the first, we shut down the engine.
			If it is the second, we Create a little window with some
			text on it. We also add a String To the list box To Log
			what happened. And If it is the third button, we Create
			a file open dialog, And add also this as String To the list box.
			That's all for the event receiver.
			EndRem
			Case EGET_BUTTON_CLICKED

				Select id
					Case GUI_ID_QUIT_BUTTON
						Context.device.closeDevice()
						Return True
	
					Case GUI_ID_NEW_WINDOW_BUTTON
						Context.listbox.addItem("Window created")
						Context.counter :+ 30
						If Context.counter > 200 Then Context.counter = 0
	
						Local window:IGUIWindow = env.addWindow( ..
							_RECTI(100 + Context.counter, 100 + Context.counter, 300 + Context.counter, 200 + Context.counter), ..
							False, .. ' modal?
							"Test window")
	
						env.addStaticText("Please close me", ..
							_RECTI(35,35,140,50), ..
							True, .. ' border?
							False, .. ' wordwrap?
							window)	
											
						Return True
	
					Case GUI_ID_FILE_OPEN_BUTTON
						Context.listbox.addItem("File open")
						env.addFileOpenDialog("Please choose a file.")
						Return True
	
					Default
						Return False
				EndSelect
			EndSelect
		EndIf

		Return False
	EndMethod
	
	' we must override the generate function in order for instantiation to work properly
	Function generate:IEventReceiver()
		Return New MyEventReceiver
	EndFunction
EndType


Rem
Ok, now For the more interesting part. First, Create the Irrlicht device. As in
some examples before, we ask the user which driver he wants To use For this
example:
EndRem

' ask user For driver

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device And Exit If creation failed

Local device:IrrlichtDevice = IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480))

If Not device Return ' could Not create selected driver.

Rem 
	The creation was successful, now we set the event receiver And
	store pointers To the driver And To the gui environment. 
EndRem

device.setWindowCaption("Irrlicht Engine - User Interface Demo")
device.setResizable(True)

Local driver:IVideoDriver = device.getVideoDriver()
Local env:IGUIEnvironment=device.getGUIEnvironment()

Rem
To make the font a little bit nicer, we Load an external font
And set it as the New Default font in the skin.
To keep the standard font For tool tip text, we set it To
the built-in font.
EndRem

Local skin:IGUISkin = env.getSkin()
Local font:IGUIFont = env.getFont("../media/fonthaettenschweiler.bmp")
If (font) Then skin.setFont(font)
skin.setFont(env.getBuiltInFont(), EGDF_TOOLTIP)

Rem
We add three buttons. The first one closes the engine. The second
creates a window And the third opens a file open dialog. The third
parameter is the id of the button, with which we can easily identify
the button in the event receiver.
EndRem	

env.addButton(_RECTI(10,240,110,240 + 32), Null, GUI_ID_QUIT_BUTTON, ..
	"Quit", "Exits Program")
env.addButton(_RECTI(10,280,110,280 + 32), Null, GUI_ID_NEW_WINDOW_BUTTON, ..
	"New Window", "Launches a new Window")
env.addButton(_RECTI(10,320,110,320 + 32), Null, GUI_ID_FILE_OPEN_BUTTON, ..
	"File Open", "Opens a file")

Rem
Now, we add a static text And a scrollbar, which modifies the
transparency of all gui elements. We set the maximum value of
the scrollbar To 255, because that's the maximal value for
a color value.
Then we Create an other static text And a list box.
EndRem

env.addStaticText("Transparent Control:", _RECTI(150,20,350,40), True)
Local scrollbar:IGUIScrollBar = env.addScrollBar(True, ..
	_RECTI(150, 45, 350, 60), Null, GUI_ID_TRANSPARENCY_SCROLL_BAR)
scrollbar.setMax(255)

' set scrollbar position To alpha value of an arbitrary element
scrollbar.setPos(env.getSkin().GetColor(EGDC_WINDOW).GetAlpha())

env.addStaticText("Logging ListBox:", _RECTI(50,110,250,130), True)
Local listbox:IGUIListBox = env.addListBox(_RECTI(50, 140, 250, 210))
env.addEditBox("Editable Text", _RECTI(350, 80, 550, 100))

' Store the appropriate data in a context structure.
Local context:SAppContext = New SAppContext
context.device = device
context.counter = 0
context.listbox = listbox

' Then Create the event receiver, giving it that context structure.
Local receiver:IEventReceiver = ..
	IEventReceiver.Create(MyEventReceiver.generate)
MyEventReceiver(receiver).Context = context

' And tell the device To use our custom event receiver.
device.setEventReceiver(receiver)

Rem
And at last, we Create a nice Irrlicht Engine logo in the top Left corner. 
EndRem
env.addImageFromTexture(driver.getTexture("../media/irrlichtlogo2.png"), ..
	_POSITION2DI(10,10))

Rem
That's all, we only have to draw everything.
EndRem

While device.run() And driver.isValid()
	If device.isWindowActive()
	
		driver.beginScene(True, True, _SCOLOR(0,200,200,200))
	
		env.drawAll()
	
		driver.endScene()
	EndIf
Wend

device.drop()
