Rem
* Example 007 Collision

We will describe 2 methods: Automatic collision detection For moving through 3d worlds
with stair climbing And sliding, And manual scene node And triangle picking using a
ray.  In this Case, we will use a ray coming out from the camera, but you can use
any ray.

To start, we take the program from tutorial 2, which loads And displays a quake
3 level. We will use the level To walk in it And To pick triangles from. In
addition we'll place 3 animated models into it for triangle picking. The
following code starts up the engine And loads a quake 3 level, as per tutorial 2.
EndRem
SuperStrict
Framework Irrlicht.Core

' I use this ISceneNode ID To indicate a scene node that is
' Not pickable by getSceneNodeAndCollisionPointFromRay()
Const ID_IsNotPickable:Int = 0

' I use this flag in ISceneNode IDs To indicate that the
' scene node can be picked by ray selection.
Const IDFlag_IsPickable:Int = 1 Shl 0

' I use this flag in ISceneNode IDs To indicate that the
' scene node can be highlighted.  In this example, the
' homonids can be highlighted, but the level mesh can't.
Const IDFlag_IsHighlightable:Int = 1 Shl 1

' let user Select driver Type

Local driverType:Int = EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480), 16, False)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

device.getFileSystem().addZipFileArchive("../media/map-20kdm2.pk3")

Local q3levelmesh:IAnimatedMesh = smgr.getMesh("20kdm2.bsp")

Local q3node:ISceneNode

' The Quake mesh is pickable, but doesn't get highlighted.
If (q3levelmesh) Then q3node = smgr.addOctreeSceneNode(q3levelmesh.getMesh(0), Null, IDFlag_IsPickable)

Rem
So far so good, we've loaded the quake 3 level like in tutorial 2. Now,
here comes something different: We Create a triangle selector. A
triangle selector is a class which can fetch the triangles from scene
nodes For doing different things with them, For example collision
detection. There are different triangle selectors, And all can be
created with the ISceneManager. In this example, we Create an
OctTreeTriangleSelector, which optimizes the triangle output a little
bit by reducing it like an octree. This is very useful For huge meshes
like quake 3 levels. After we created the triangle selector, we attach
it To the q3node. This is Not necessary, but in this way, we do Not
need To care For the selector, For example dropping it after we do Not
need it anymore.
EndRem

Local selector:ITriangleSelector
	
If (q3node)
	q3node.setPosition(_VECTOR3DF(-1350,-130,-1400))

	selector = smgr.createOctreeTriangleSelector( ..
		q3levelmesh.getMesh(0), q3node, 128)
	q3node.setTriangleSelector(selector)
	' We're not done with this selector yet, so don't drop it.
EndIf

Rem
We add a first person shooter camera To the scene so that we can see And
move in the quake 3 level like in tutorial 2. But this, time, we add a
special animator To the camera: A Collision Response animator. This
animator modifies the scene node To which it is attached To in order To
prevent it moving through walls, And To add gravity To it. The
only thing we have To tell the animator is how the world looks like,
how big the scene node is, how much gravity To apply And so on. After the
collision response animator is attached To the camera, we do Not have To do
anything more For collision detection, anything is done automatically.
The rest of the collision detection code below is For picking. And please
note another cool feature: The collision response animator can be
attached also To all other scene nodes, Not only To cameras. And it can
be mixed with other scene node animators. In this way, collision
detection And response in the Irrlicht engine is really easy.

Now we'll take a closer look on the parameters of
createCollisionResponseAnimator(). The first parameter is the
TriangleSelector, which specifies how the world, against collision
detection is done looks like. The second parameter is the scene node,
which is the Object, which is affected by collision detection, in our
Case it is the camera. The third defines how big the Object is, it is
the radius of an ellipsoid. Try it out And change the radius To smaller
values, the camera will be able To move closer To walls after this. The
Next parameter is the direction And speed of gravity.  We'll set it to
(0, -10, 0), which approximates To realistic gravity, assuming that our
units are metres. You could set it To (0,0,0) To disable gravity. And the
last value is just a translation: Without this, the ellipsoid with which
collision detection is done would be around the camera, And the camera would
be in the middle of the ellipsoid. But as human beings, we are used To have our
eyes on top of the body, with which we collide with our world, Not in
the middle of it. So we place the scene node 50 units over the center
of the ellipsoid with this parameter. And that's it, collision
detection works now.
EndRem

' Set a jump speed of 3 units per second, which gives a fairly realistic jump
' when used with the gravity of (0, -10, 0) in the collision response animator.
Local camera:ICameraSceneNode = ..
	smgr.addCameraSceneNodeFPS(Null, 100.0, .3, ID_IsNotPickable, Null, True, 3)

camera.setPosition(_VECTOR3DF(50,50,-60))
camera.setTarget(_VECTOR3DF(-70,30,-60))

If selector Then
	Local anim:ISceneNodeAnimator = smgr.createCollisionResponseAnimator( ..
		selector, camera, _VECTOR3DF(30,50,30), ..
		_VECTOR3DF(0,-10,0), _VECTOR3DF(0,50,0))
	selector.drop() ' As soon as we're done with the selector, drop it.
	camera.addAnimator(anim)
	anim.drop()
EndIf 

' Now I Create three animated characters which we can pick, a dynamic light For
' lighting them, And a billboard For drawing where we found an intersection.

' First, let's get rid of the mouse cursor.  We'll use a billboard to show
' what we're looking at.
device.getCursorControl().setVisible(False)

' Add the billboard.
Local bill:IBillboardSceneNode = smgr.addBillboardSceneNode()
bill.setMaterialType(EMT_TRANSPARENT_ADD_COLOR )
bill.setMaterialTexture(0, driver.getTexture("../media/particle.bmp"))
bill.setMaterialFlag(EMF_LIGHTING, False)
bill.setMaterialFlag(EMF_ZBUFFER, False)
bill.setSize(_DIMENSION2DF(20.0, 20.0))
bill.setID(ID_IsNotPickable) ' This ensures that we don't accidentally ray-pick it

Rem Add 3 animated hominids, which we can pick using a ray-triangle intersection.
They all Animate quite slowly, To make it easier To see that accurate triangle
selection is being performed.
EndRem
Local node:IAnimatedMeshSceneNode

' Add an MD2 node, which uses vertex-based animation.
node = smgr.addAnimatedMeshSceneNode(smgr.getMesh("../media/faerie.md2"), ..
					Null, IDFlag_IsPickable | IDFlag_IsHighlightable) 
node.setPosition(_VECTOR3DF(-70,-15,-120)) ' Put its feet on the Floor.
node.SetScale(_VECTOR3DF(2, 2, 2)) ' Make it appear realistically scaled
node.setMD2Animation(EMAT_POINT)
node.setAnimationSpeed(20)

Local material:SMaterial=SMaterial.Create()
material.setTexture(0,driver.getTexture("../media/faerie2.bmp"))
material.setLighting(True)
material.setNormalizeNormals(True)
node.setMaterial(0, material)

' Now Create a triangle selector For it.  The selector will know that it
' is associated with an animated node, And will update itself as necessary.
selector = smgr.createTriangleSelectorFromAnimatedNode(node)
node.setTriangleSelector(selector)
selector.drop() ' We're done with this selector, so drop it now.

' This X files uses skeletal animation, but without skinning.
node = smgr.addAnimatedMeshSceneNode(smgr.getMesh("../media/dwarf.x"), ..
					Null, IDFlag_IsPickable | IDFlag_IsHighlightable)
node.setPosition(_VECTOR3DF(-70,-66,0)) ' Put its feet on the Floor.
node.SetRotation(_VECTOR3DF(0,-90,0)) ' And turn it towards the camera.
node.setAnimationSpeed(20)
selector = smgr.createTriangleSelectorFromAnimatedNode(node)
node.setTriangleSelector(selector)
selector.drop()

' And this B3D file uses skinned skeletal animation.
node = smgr.addAnimatedMeshSceneNode(smgr.getMesh("../media/ninja.b3d"), ..
					Null, IDFlag_IsPickable | IDFlag_IsHighlightable)
node.SetScale(_VECTOR3DF(10, 10, 10))
node.setPosition(_VECTOR3DF(-70,-66,-60))
node.SetRotation(_VECTOR3DF(0,90,0))
node.setAnimationSpeed(10)
node.getMaterial(0).setNormalizeNormals(True)
' Just do the same as we did above.
selector = smgr.createTriangleSelectorFromAnimatedNode(node)
node.setTriangleSelector(selector)
selector.drop()

material.setTexture(0, Null)
material.setLighting(False)

' Add a light, so that the unselected nodes aren't completely dark.
Local light:ILightSceneNode = smgr.addLightSceneNode(Null, _VECTOR3DF(-60,100,400), ..
	_SCOLORF(1.0,1.0,1.0,1.0), 600.0)
light.setID(ID_IsNotPickable) ' Make it an invalid target For selection.

' Remember which scene node is highlighted
Local highlightedSceneNode:ISceneNode
Local collMan:ISceneCollisionManager = smgr.getSceneCollisionManager()
Local lastFPS:Int = -1

' draw the selection triangle only as wireframe
material.setWireframe(True)

While(device.run())
	If (device.isWindowActive())

		driver.beginScene(True, True, Null)
	
		smgr.drawAll()
		
		' Unlight any currently highlighted scene node
		If highlightedSceneNode
			highlightedSceneNode.setMaterialFlag(EMF_LIGHTING, True)
			highlightedSceneNode = Null
		EndIf

		' All intersections in this example are done with a ray cast out from the camera To
		' a distance of 1000.  You can easily modify this To check (e.g.) a bullet
		' trajectory Or a sword's position, or create a ray from a mouse click position using
		' ISceneCollisionManager::getRayFromScreenCoordinates()
		Local ray:Line3df = Line3df.Create()		
		ray.getStart().setEq(camera.getPosition())
		ray.getEnd().setEq(ray.getStart().Plus(camera.getTarget().Minus(ray.getStart()).normalize().Mult(1000.0)))
	
		' Tracks the current intersection point with the level Or a mesh
		Local intersection:Vector3df = _VECTOR3DF()
		' Used To show with triangle has been hit
		Local hitTriangle:Triangle3df = Triangle3df.Create()
		
		' This call is all you need To perform ray/triangle collision on every scene node
		' that has a triangle selector, including the Quake level mesh.  It finds the nearest
		' collision point/triangle, And returns the scene node containing that point.
		' Irrlicht provides other types of selection, including ray/triangle selector,
		' ray/box And ellipse/triangle selector, plus associated helpers.
		' See the methods of ISceneCollisionManager
		Local selectedSceneNode:ISceneNode = ..
			collMan.getSceneNodeAndCollisionPointFromRay( ..
					ray, ..
					intersection, .. ' This will be the position of the collision
					hitTriangle, .. ' This will be the triangle hit in the collision
					IDFlag_IsPickable, .. ' This ensures that only nodes that we have
							.. ' set up To be pickable are considered
					Null) ' Check the entire scene (this is actually the implicit Default)

		' If the ray hit anything, move the billboard To the collision position
		' And draw the triangle that was hit.
		If selectedSceneNode
			bill.setPosition(intersection)

			' We need To reset the transform before doing our own rendering.
			driver.SetTransform(ETS_WORLD, Matrix4.Create())
			driver.setMaterial(material)
			driver.draw3DTriangle(hitTriangle, _SCOLOR(0,255,0,0))
			
			' We can check the flags For the scene node that was hit To see If it should be
			' highlighted. The animated nodes can be highlighted, but Not the Quake level mesh
			If ((selectedSceneNode.getID() & IDFlag_IsHighlightable) = IDFlag_IsHighlightable)
				highlightedSceneNode = selectedSceneNode
	
				' Highlighting in this Case means turning lighting OFF For this node,
				' which means that it will be drawn with full brightness.
				highlightedSceneNode.setMaterialFlag(EMF_LIGHTING, False)
			EndIf
		EndIf

		' We're all done drawing, so end the scene.
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
		
		If (lastFPS <> fps)
			Local str:String = "Collision detection example - Irrlicht Engine ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
EndWhile
	
device.drop()
